

original_tty_state=$(stty -g)
dcsOnOff=false      # Turns the DCS on (off) before (after) running all the scans
quietMode=0         # Trims the output to the DB-related info + the scan results
createConfig=false  # Creates / overwrites Yarr config files based on config.json
dcsMonitor=""       # 

function wrapAndExit(){
    echo -e "\n-------------------------"
    echo "[ info ][so] Stoped by SIGINT"

    echo "[ info ][so] Reverting the terminal to its original tty state..."
    stty $original_tty_state

    if [[ "$dcsMonitor" != "" ]]; then
        echo "[ info ][so] waiting for the dcs monitor script to finish..."
        echo "0" > $SOBASE/.counters/dcsStat
        waitForItToFinish "read_dcs_backgr"
    fi
     
    echo "[ info ][so] -- done -- "
    exit 1
}

trap 'wrapAndExit' 2

function waitForItToFinish(){
    while [ "$(pgrep -x "$1")" != "" ]
    do
        sleep 0.1
    done
}

function usage(){
    cat <<EOF
$0
Help:
  -j [str, path] : path to json configuration file
  -m [str]       : Serial number of the module
  -W             : upload Yarr's results to localDB (scanConsole's -W)
  -Q             : QC mode (upload / retrieve from iTk PD, etc) (scanConsole's -Q).
  -o             : turn on / off the output of the PS before and after all the scans
  -d [str, path] : powersupply.json. Monitor the PS and upload the DCS data to influxDB.
  -t [str, path] : influxdb_connectivity.json. Move DCS data from InfluxDB to LocalDB after each scan.
  -c             : create / override Yarr's config files for your chip based on config.json.
  -q [int]       : "quiet" mode (cleaner output), 0, or 1. Default 0
EOF
    exit 1
}
while getopts m:j:q:d:t:ocWQ OPT
do
    case $OPT in
        m) module_id=$OPTARG ;;
        j) CFGFILE=$(realpath $OPTARG) ;;
        q) quietMode=$OPTARG ;;
        t) syncOption="-t $OPTARG" ;;
        o) dcsOnOff=true ;;
        d) dcsMonitor="-d $OPTARG" ;;
        c) createConfig=true ;;
        W) localdbOpt="-W" ;;
        Q) qcOpt="-Q" ;;
        *) usage ;;
    esac
done

echo "--------------------------------"

if [ -z $CFGFILE ];then
    echo "[ error ][so] You didn't specify the json cfg file."
    echo "[ error ][so] The default one is under configs/config.json"
    usage
    exit
fi

export YARRBASE=`jq -j '.YARR.YARR_directory' $CFGFILE`
export CFGFILE=$CFGFILE
export SOBASE=$(cd $(dirname $0); pwd)

mkdir -p $SOBASE/.counters
touch $SOBASE/.counters/runNumber $SOBASE/.counters/today
if ! [[ "$(<$SOBASE/.counters/runNumber)" =~ ^[0-9]+$ ]]; then
    echo "0" > $SOBASE/.counters/runNumber
fi
if [[ $(date "+%y%m%d") -gt $(<$SOBASE/.counters/today) ]]; then
    echo $(date "+%y%m%d") > $SOBASE/.counters/today
    echo "0" > $SOBASE/.counters/runNumber
fi


echo "$(($(<$SOBASE/.counters/runNumber)+1))" > $SOBASE/.counters/runNumber

export SO_RUNNUMBER="$(<$SOBASE/.counters/today)_$(printf "%03d" $(<$SOBASE/.counters/runNumber))"

mkdir -p $SOBASE/data/$SO_RUNNUMBER

if [ ! -d "$YARRBASE" ]; then
    echo "[ error ][so] Please specify a valid path to Yarr under\n"
                  ".YARR.YARR_directory in the main config file"
    exit 1
fi

if [ $dcsOnOff ] || [[ "$dcsMonitor" != "" ]]
then
    lrLibPath=`jq -j '.dcs_control.LR_lib_path' $CFGFILE`
    export PYTHONPATH=$lrLibPath:$PYTHONPATH
fi

if [[ "$dcsMonitor" != "" ]]
then
    chn=""

    nChannels="$(jq -j '.dcs_control.channels | length' $CFGFILE)"
    for (( i=0; i<$nChannels; i++ ))
    do
        chName="$(jq -j ".dcs_control.channels[$i].name" $CFGFILE)"
        chn+="$chName,"
    done
    chn="${chn::-1}"
    
    # Perform a first DCS Measurement to check for possible authentication
    while true; do 
        python3 $SOBASE/libDCS/qaqc.py -e $(echo $dcsMonitor | cut -d' ' -f2) -c $chn -d $(echo $dcsMonitor | cut -d' ' -f2) measure
        rv=$?
        if [ $rv -eq 0 ]; then 
            # No authentication / No errors
            break

        elif [ $rv -eq 1 ]; then 
            # Error othen than authentication failure
            exit 1; 

        elif [ $rv -eq 2 ]; then

            # Authentication failure
            echo -n "Password: " 
            read -s pwd
            export INFLUXDBPWD=$pwd
            echo ""
        fi
    done
fi

if [[ "$syncOption" == "" ]] && [[ "$localdbOpt" != "" ]]
then
    syncScript=$SOBASE/data/$SO_RUNNUMBER/moveDCStoLocalDB.sh
    touch $syncScript
    echo "# Usage: " >> $syncScript
    echo "#     bash moveDCStoLocalDB.sh influxdb_connectivity.json" >> $syncScript
    echo -e "\n" >> $syncScript
fi


if [ -z $module_id ];then
    echo "You didn't specify the Module ID (option '-m')."
    usage
fi

value="$(cat $CFGFILE | jq --arg a $module_id '.modules."\($a)"')"
if [ "$value" = "null" ]; then
    echo "No module ID \"$module_id\" found under the entry \"modules\" in $CFGFILE. aborting..."
    exit
fi

if $createConfig ; then
    cfgdir=$SOBASE/configs/rd53a/$module_id
    rm -rf $cfgdir
    echo "[ info ][so] Creating configs for module \"$module_id\""

    cmd="$YARRBASE/bin/dbAccessor -D -n $module_id -p $cfgdir"
    $cmd
    retval=$?
    if [ $retval -ne 0 ]; then
        if [ "$qcOpt" = "-Q" ]; then
            exit
        else
            rm -rf $cfgdir
            echo "Do you continue scan with temporary config files?"

            unset answer
            while [ -z "${answer}" ]; do
                read -p "[y/n]: " answer
            done
            if [[ ${answer} != "y" ]]; then
                echo "Aborting..."
                exit
            fi
            conntmp="/tmp/$RANDOM"
            conn="{\"chipType\":\"$chiptype\",\"chips\":[]}"
            chiptype=$(jq -j --arg a $module_id '.modules."\($a)".chipType' $CFGFILE)
            if [ "$chiptype" != "null" ]; then
                conn=$(echo $conn | jq ".chipType |= .+\"$chiptype\"")
            fi
            nChips="$(jq -j --arg a $module_id '.modules."\($a)".chips | length' $CFGFILE)"
            cnt=0
            while [ $cnt -lt $nChips ]; do

                # "name" in connectivity.json has to match the actual chipName
                if [ "$module_id" != "null" ]; then
                    conn=$(echo $conn | jq ".chips[$cnt] |= .+{\"name\":\"$module_id\"}")
                fi

                # these parameters will be modified later
                conn=$(echo $conn | jq ".chips[$cnt] |= .+{\"tx\":0}")
                conn=$(echo $conn | jq ".chips[$cnt] |= .+{\"rx\":0}")
                conn=$(echo $conn | jq ".chips[$cnt] |= .+{\"enable\":1}")
                conn=$(echo $conn | jq ".chips[$cnt] |= .+{\"locked\":0}")

                cnt=$(( cnt + 1 ))

            done
            echo $conn | jq . > $conntmp

            cmd="$YARRBASE/bin/dbAccessor -D -c $conntmp -p $cfgdir"
            $cmd
            retval=$?
            rm $conntmp
            if [ $retval -ne 0 ]; then
                exit
            fi
        fi
    fi
    echo "[ info ][so] Creating config file for module \"$module_id\": $cfgdir/$module_id.json"
    $YARRBASE/bin/createConfig -t RD53A -n $module_id -o $cfgdir/$module_id.json


    nChips="$(jq -j --arg b ${module_id} '.modules."\($b)".chips | length' $CFGFILE)"
    for (( i=0; i<$nChips; i++ ))
    do

        # Modify retrieved Yarr config files according to config.json

        # Chip config file 
        echo ""

        JSON_FILE=$cfgdir/$module_id.json
        jq -r --argjson a ${i} --arg b ${module_id} '.modules."\($b)".chips[$a].GlobalConfig | keys[] as $k | "\($k) \(.[$k])"' $CFGFILE |
        while IFS=" " read -r -a value; do
            key=${value[0]}
            val=${value[1]}

            currentValue=$(jq --arg k ${key} '.RD53A.GlobalConfig."\($k)"' $JSON_FILE)

            if [ "$currentValue" == "null" ];
            then
                echo "[ warn ] Didn't find key \"$key\" in $JSON_FILE"
            else
                jq --argjson v $val --arg k $key '.RD53A.GlobalConfig."\($k)" = $v' $JSON_FILE > currentValue && mv currentValue $JSON_FILE
                echo "[ info ] Changed \"$key\" from \"$currentValue\" to \"$val\" in file $JSON_FILE"
            fi
        done

        # Connectivity file:
        echo ""
        CNCT_FILE=$cfgdir/connectivity.json
        jq -r --argjson a ${i} --arg b ${module_id} '.modules."\($b)".chips[$a].connectivity | keys[] as $k | "\($k) \(.[$k])"' $CFGFILE |
        while IFS=" " read -r -a value; do
            key=${value[0]}
            val=${value[1]}

            currentValue=$(jq --argjson a ${i} --arg k ${key} '.chips[$a]."\($k)"' $CNCT_FILE)
            if [ "$currentValue" == "null" ];
            then
                echo "[ warn ][so] Didn't find key \"$key\" in $CNCT_FILE"
            else
                jq --argjson a ${i} --argjson v $val --arg k $key '.chips[$a]."\($k)" = $v' $CNCT_FILE > currentValue && mv currentValue $CNCT_FILE
                echo "[ info ][so] Changed \"$key\" from \"$currentValue\" to \"$val\" in file $CNCT_FILE (chip $i)"
            fi
        done

        # Create an entry with the module serial number (to associate the config
        # files to in localdb)
        jq '. += {"module"}' $CNCT_FILE > tmp && mv tmp $CNCT_FILE
        jq --arg a ${module_id} '.module += {"serialNumber": "\($a)"}' $CNCT_FILE > tmp && mv tmp $CNCT_FILE
        jq '.module += {"componentType": "module"}' $CNCT_FILE > tmp && mv tmp $CNCT_FILE
        
    done

    # Controller file
    CRTL_FILE=$cfgdir/controller.json 
    cp $YARRBASE/configs/controller/specCfg.json $CRTL_FILE 
    specNum=$(jq -j --arg b ${module_id} '.modules."\($b)".specNum' $CFGFILE)
    if [ "$specNum" != "null" ]; then
        jq --argjson v $specNum '.ctrlCfg.cfg.specNum = $v' $CRTL_FILE > currentValue && mv currentValue $CRTL_FILE
    fi

    echo "------------------------"
    cat $cfgdir/connectivity.json | jq .
    echo "------------------------"
    echo "[ warn ][so] Are these settings correct?"
    unset answer
    while [ -z "${answer}" ]; do
        read -p "[y/n]: " answer
    done
    if [[ ${answer} != "y" ]]; then
        echo "[ error ][so] Please modify the configuration in $CFGFILE and try again."
        exit
    fi

fi

if $dcsOnOff ;then
    echo -e "\n[ info ][so] Turning ON the DCS..."

    nChannels="$(jq -j '.dcs_control.channels | length' $CFGFILE)"
    for (( i=0; i<$nChannels; i++ ))
    do
        chn="$(jq -j ".dcs_control.channels[$i].name" $CFGFILE)"
        V="$(jq -j ".dcs_control.channels[$i].voltage" $CFGFILE)"
        I="$(jq -j ".dcs_control.channels[$i].current" $CFGFILE)"
        python3 $SOBASE/libDCS/qaqc.py -e $(echo $dcsMonitor | cut -d' ' -f2) -c $chn -d $(echo $syncOption | cut -d' ' -f2) power-on $V $I
        if [ $? -ne 0 ]; then exit 1; fi
    done
fi

echo -e "\n[ info ][so] Running the sequence of scans"


nScans="$(jq -j '.YARR.scan_list | length' $CFGFILE)"
for i in $(seq 1 $nScans); do

    if [[ "$dcsMonitor" != "" ]]; then waitForItToFinish "read_dcs_backgr"; fi

    obj="$(jq -j  --argjson a ${i} '.YARR.scan_list[$a-1]' $CFGFILE)"
    obj="$(echo -e "${obj}" | tr -d '[:space:]')"
    
    comm="$SOBASE/scanLauncher.sh -m ${module_id} -s $obj ${localdbOpt} ${qcOpt} ${syncOption} ${dcsMonitor}"
    echo $comm
    echo -e "[ info ] |\n[ info ] | - $(echo $obj | jq -r '.[0]')\n[ info ] |"
    if [ $quietMode -gt 0 ];
    then
        output=$($comm 2>&1 | awk '/ Mean /{print $0}')
        retval=$?
        if [ "$output" != "" ];then
            while IFS= read -r line ; do echo -e "[ info ] |     -> ${line##*\] }"; done <<< "$output"
        else
            echo -e "[ info ] |     -> done"
        fi
    else
        $comm
        retval=$?
    fi
    if [ $retval -ne 0 ]; then
        echo "[ info ][so] Wrapping and exiting after receiving an error"
        echo "0" > $SOBASE/.counters/dcsStat
        break
    fi
done



if $dcsOnOff; then
    echo -e "\n[ info ][so] Turning OFF the DCS..."

    nChannels="$(jq -j '.dcs_control.channels | length' $CFGFILE)"
    for (( i=0; i<$nChannels; i++ ))
    do
        chn="$(jq -j ".dcs_control.channels[$i].name" $CFGFILE)"
        python3 $SOBASE/libDCS/qaqc.py -e $(echo $dcsMonitor | cut -d' ' -f2) -c $chn -d $(echo $syncOption | cut -d' ' -f2) power-off
        if [ $? -ne 0 ]; then exit 1; fi
    done
fi


if [[ "$dcsMonitor" != "" ]]; then

    echo "[ info ][so] waiting for the dcs monitor script to finish..."
    waitForItToFinish "read_dcs_backgr"
fi

if [[ "$syncOption" == "" ]] && [[ "$localdbOpt" != "" ]]
then

cat << EOF
[ info ][so] If you want to upload to localDB the DCS data associated
             to all the scans you have just run (assuming you have it 
             stored in InfluxDB), you can do it in one command:

                 bash $SOBASE/data/$SO_RUNNUMBER/moveDCStoLocalDB.sh path/to/influxdb_connectivity.json
EOF

fi


