#!/bin/bash
#set -x

########################################
# Author : Shohei Yamagaya(Osaka univ.)
########################################

function wrapAndExit(){
    exit 1
}

trap 'wrapAndExit' 2

function usage(){
    cat <<EOF
$0
Help:
  -m <str>      : Serial number of module (Req *)
  -s <str/path> : scan type               (Req *)
  -d <str/path> : enable the DCS functinality
  -t            : Sync DCS data from InfluxDB to LocalDB after each scan.
  -W            : upload Yarr's results to localDB
  -Q            : enable file valification and interactive features for QC testing.
  -h            : Show this
EOF
    exit 1
}

function check_value(){
    if [[ "$2" == "" ]] || [[ $2 = -* ]];then
        echo "[ error ][sl] option \"$1\" requires a parameter!"
	    usage
    fi
}

###### default ##############################
dcsMonitor=false
syncDatabases=false
powersupplyConfig=""
dbComConfig=""
localdbOpt=""
#############################################

while getopts m:s:d:t:WQh OPT
do
    case $OPT in
    m) module_id=$OPTARG ;;
    s) scan=$OPTARG ;;
    d) dcsMonitor=true; powersupplyConfig=$OPTARG ;;
    t) syncDatabases=true; dbComConfig=$OPTARG ;;
    W) localdbOpt="-W" ;;
    Q) qcOpt="-Q" ;;
    h) usage;;
    *) usage ;;
    esac
done

# Check if these oprions where properly supplied
check_value "-m" $module_id
check_value "-s" $scan
if $dcsMonitor    ; then check_value "-d" $powersupplyConfig; fi
if $syncDatabases ; then check_value "-t" $dbComConfig      ; fi


scanName=$(echo $scan | jq -r '.[0]')
scanPath="$YARRBASE/configs/scans/rd53a/$scanName.json"

# Build Yarr's "-t" option
if echo "${scanName}" | grep "tune_globalpreamp" >>/dev/null; then
    if [[ $(echo $scan | jq '. | length') == 1 ]]; then
        echo "[ error ] Please specify the ToT together with the scan name"
        exit 1
    else
        targetAmpOrCharge=$(jq -j '.YARR.scan.target_preamp' $CFGFILE)
        targetToT=$(echo $scan | jq -r '.[1]')
    fi
else
    targetToT=""
    if [[ $(echo $scan | jq '. | length') == 2 ]]; then
        targetAmpOrCharge=$(echo $scan | jq -r '.[1]')
    else
        targetAmpOrCharge=""
    fi
fi

###### Log file  ######################
RunNumber=$(cat $HOME/.yarr/runCounter)
RunNumber=$(( RunNumber +  1))
printf -v RunNumber "%06d" $RunNumber
#export result_data_DIR=$SOBASE"/data/"$RunNumber"_"${scanName}"/"
#mkdir -p $result_data_DIR
#LOGFILE="${result_data_DIR}OperatorLog."`date "+%Y%m%d_%H%M%S"`
###############################################


if ${dcsMonitor} ;then
    echo "[ info ][sl] Getting single [V, I] measurement"
    cfg="$powersupplyConfig"
    chn=""

    nChannels="$(jq -j '.dcs_control.channels | length' $CFGFILE)"
    for (( i=0; i<$nChannels; i++ ))
    do
        chName="$(jq -j ".dcs_control.channels[$i].name" $CFGFILE)"
        chn+="$chName,"
    done
    chn="${chn::-1}"
    python3 $SOBASE/libDCS/qaqc.py -e $cfg -c $chn -d $cfg measure

    # Tell read_dcs_background.sh that we're performing a scan
    echo "1" > $SOBASE/.counters/dcsStat
    $SOBASE/libDCS/read_dcs_background.sh $cfg $cfg &
fi
targetCharge=$(jq -j '.YARR.scan.target_charge' $CFGFILE)

ctrlFile="$SOBASE/configs/rd53a/$module_id/controller.json"
cnctFile="$SOBASE/configs/rd53a/$module_id/connectivity.json"

comm="$YARRBASE/bin/scanConsole -r $ctrlFile -c $cnctFile -p -t ${targetAmpOrCharge} ${targetToT} -s ${scanPath} -m 1 $localdbOpt $qcOpt -o $SOBASE/data/$SO_RUNNUMBER" #> /dev/null 2>1
echo "[ info ][sl] Calling Yarr's scanConsole"
echo "[ info ][sl] $comm"


$comm
retval=$?
if [ $retval -ne 0 ]; then
    exit $retval
fi

# stop taking data
if ${dcsMonitor} ;then
    echo "[ info ][sl] Stopping data taking"
    echo "0" > $SOBASE/.counters/dcsStat
fi

if [[ "$localdbOpt" != "" ]]; then

    if [ $dcsMonitor ] && ${syncDatabases} ;then

        cmd="$YARRBASE/bin/dbAccessor -F $(readlink -f $dbComConfig) -n $module_id -s $SOBASE/data/$SO_RUNNUMBER/${RunNumber}_$scanName/scanLog.json -d $(readlink -f $dbComConfig)"
        echo "[ info ][sl] Pushing the DCS data to localDB"
        echo "[ info ][sl] $cmd"
        $cmd
        echo "[ info ][sl] done"

    elif ! ${syncDatabases} ;then

        cmd="$YARRBASE/bin/dbAccessor -F \$1 -n $module_id -s $SOBASE/data/$SO_RUNNUMBER/${RunNumber}_$scanName/scanLog.json -d \$1"
        echo "$cmd" >> $SOBASE/data/$SO_RUNNUMBER/moveDCStoLocalDB.sh

    fi
fi

